function [Uni,UniD,Gamma,GammaD,logProb,logProbD,PostGamD,MargGam,Ybar,dbar, ...
Ystack,dstack]= bvsme_prob(type,h,X,Z,nmetr,r,Hg,w,burnin)

%bvsme_prob: Bayesian Variable Selection with Metropolis search,
% 	     selection prior with diagonal cov matrix 
%           and fast form of QR matrices updating
%			     Main Program 
%***********************************************************************
%INPUTS:
% type		0 for nominal response, 1 for ordinal response
% h		hyperparameter 
% X,Z		training set, matrix X(n by p) and vector Z of class labels
% nmetr		total number of iterations
% r,      	starting binary vector with r elements equal to 1 
% Hg,   	hyperparameter of the prior on regression coefficients
%		- vector of std's
% w,		hyperparameter of prior on latent binary vector
%		- vector of parameters w_j, j=1...p (Bernoulli priors)
%		- scalar in the case of equal parameters w_j=w 
%		  (Binomial prior with mean pw)
%		- vector of two values for a Beta-Binomial prior
% 		(with mean=pw(1), w(2)=sum of the two parameters of the Beta)
% burnin	number of iteration for burnin	
%
%OUTPUTS:
% Uni,		vector with number of ones of visited models
% UniD,		vector with number of ones of distinct visited models
% GammaD,	distinct visited models, ordered according to
%		their (normalized) relative posterior probability
%		(cell array of strings)
% logProb,      log-relative posterior probabilities 
%		of all visited models
% logProbD      log-relative posterior probabilities
%               of distinct visited models
% PostGamD	normalized ordered relative probabilities 
%		of distinct visited models 
% MargGam       marginal posterior probabilities
% Ybar,dbar
%
%USAGE:
% [Uni,UniD,Gamma,GammaD,logProb,logProbD,PostGamD,MargGam,Ybar,dbar, ...
% Ystack,dstack]= bvsme_prob(type,h,X,Z,nmetr,r,Hg,w,burnin)
% [Uni,UniD,GammaD,logProb,logProbD,PostGamD,MargGam,Ybar,dbar]= ...
%	bvsme_prob(type,h,X,Z,nmetr,r,Hg,w,burnin)
%
%NOTES:
% No need to save Gamma, Ystack and dstack if you do not plan to do
% cross-validation prediction (also comment the two lines as described below in the code).
% The code implements a fast form of updating QR matrices
% QRdelete-QRinsert are used to update QR matrices
% QR matrices recomputed every 500 iterations
% GammaD is in cell array format 
%  (elements are strings --> required by unique.m)
% Search for replicates among visited accepted models of same size
% Functions called: gofg_prob, metro_prob, replichef, probordf
% Ystack is better saved as cell because it is a list of matrices for q>1
%
%REFERENCE: 
%
% Sha, N., Vannucci, M., Brown, P.J., Trower, M.K., Amphlett, G. and 
% Falciani, F. (2003). Gene selection in arthritis classification with 
% large-scale microarray expression profiles. Comparative and Functional 
% Genomics, 4(2), 171-181.
%
% Sha, N., Vannucci, M., Tadesse, M.G., Brown, P.J., Dragoni, I., Davies, N., 
% Roberts, T.C., Contestabile, A., Salmon, N., Buckley, C. and Falciani, F. 
% (2004). Bayesian variable selection in multinomial probit models to identify 
% molecular signatures of disease stage. Biometrics, 60(3), 812-819.
%
%Copyright (c) 1997-2004 Marina Vannucci.
%**************************************************************************

[nOss,p]=size(X);  % p independent variables, q responses, nOss observations

if type==0
	q=length(unique(Z))-1
	if q==1 
		k=[];
		delta=[];
	else 
		k=1;
		delta=3;
	end
else
	ncat=length(unique(Z));
	q=1;
	k=[];
	delta=[];
end
pPar=1/2;
nPar=500;

%	disp('-- Starting binary vector with r elements equal to 1');
randp=randperm(p);
gamprec=zeros(1,p);
onenum=randp(1:r);
gamprec(onenum)=1;

disp(' ')
disp('------- RUNNING METROPOLIS ...')
disp(' ')

if (size(Hg,1)==1 & size(Hg,2)==1)
%--------------------------- Hg scalar - gPrior - no augmenting/fast updating

	inidim=[];
	Y=starty(type,q,Z);
	Ytilde=Y;
else
%--------------------------- Hg vector of std's - augmenting and fast updating
	inidim=max(20,length(find(gamprec)));
	Y=starty(type,q,Z);
	Ytilde=cat(1,Y, zeros(inidim,q));			 		
end

% Comment next line if you do not want to do cross-validation prediction
Gamma{1}=num2str(find(gamprec));

GammaD{1}=num2str(find(gamprec));
Ystack{1}=Y;
if type==0
	dpar=[];
	dstack=[];
else
	dparm=unifrnd(0,1000,1,ncat-2);
	dpar=[-Inf 0 sort(dparm) Inf];
	dstack{1}=dpar;
end
Uni(1)=r;
UniD(1)=r;
logProb=zeros(1,nmetr+1);
AD=[];
S=[];
conta=1;
contaD=1;
while conta<nmetr+1

%-------------------- updating QR decomposition

	[probprec, Qqr, Rqr,IRES1,IRES2] = gofg_prob(...
	h,gamprec,inidim,1,X,Ytilde,[],[],[],[],[],k,delta,w,Hg);

	logProb(conta)= probprec;
	logProbD(contaD)=probprec;

%-------------------- Metropolis (nPar iterations)

       	[YSt,UniSt,ProbSt, GammaSt, storeAD, storeS,dparS]= metro_prob(type,Z,h,...
	inidim,gamprec,probprec,X,Ytilde,Qqr,Rqr,IRES1,IRES2,w,Hg,nPar,pPar,...
	k,delta,dpar);

%Comment next line if no cross-validation
	Gamma=cat(1,Gamma,GammaSt);

	Ystack=cat(1,Ystack,YSt);
	if type==0
	else
		dstack=cat(1,dstack, dparS);	
	end
	logProb(conta+1:conta+nPar)=ProbSt;
	Uni=[Uni UniSt];
	indiciD=sort([storeAD storeS]);
	GammaD=cat(1,GammaD, GammaSt(indiciD));
	logProbD=[logProbD, ProbSt(indiciD)];	
	UniD=[UniD, UniSt(indiciD)];
	AD=cat(2, AD, (conta-1)+storeAD);
	S=cat(2, S, (conta-1)+storeS);
	conta=conta+nPar;
	contaD=length(logProbD);
	gamprec=zeros(1,p);
	gamprec(str2num(GammaSt{nPar}))=ones(1,length(str2num(GammaSt{nPar})));
	disp(' ')
       	disp('Number of iterations:')
       	disp(conta-1)
       	disp('...')
	if (size(Hg,1)==1 & size(Hg,2)==1) %--- Hg g-prior
		Ytilde=YSt{nPar};
	else
%-------------------- Update of dimensions for IRES1 and IRES2	
		media=ceil(mean(UniSt));
		inidim=max(media,length(find(gamprec)));
		Ytilde=cat(1,YSt{nPar}, zeros(inidim,q));
	end
	if type==0
	else
		dpar=dparS{nPar};
	end
end
disp(' ')
disp('------- Searching for replicates ...')
disp(' ')

%------------------- Search for replications

burnAD=find(AD<=burnin);
burnS=find(S<=burnin);
UniD(1:length(burnAD)+length(burnS))=[];
logProbD(1:length(burnAD)+length(burnS))=[];
GammaD(1:length(burnAD)+length(burnS))=[];

[UniD,GammaD, logProbD]=replichef(UniD,GammaD, logProbD);

%------------------- Normalized posterior and  marginal probs.
%                    Ordering of distinct visited vectors according
%                    to probability

Ycat=cat(3,Ystack{burnin+1:nmetr+1});
for i=1:nOss
	for j=1:q
		Ybar(i,j)=mean(Ycat(i,j,:));
	end
end
if type==1 
	dbar=mean(cat(1,dstack{burnin+1:nmetr+1}));
else
	dbar=[];
end;

for i=1:length(UniD)
	gamma=zeros(1,p);
	numeri=str2num(GammaD{i});
	lgam=length(numeri);
	gamma(numeri)=ones(1,lgam);
	if (size(Hg,1)==1 & size(Hg,2)==1) %--- Hg g-prior
		Ytilde=Ybar;
		inidim=[];
	else
		Ytilde=cat(1,Ybar, zeros(lgam,q));
		inidim=lgam;
	end
	[probprec, Qqr, Rqr,IRES1,IRES2] = gofg_prob(...
	h,gamma,inidim,1, X,Ytilde,[],[],[],[],[],k,delta,w,Hg);		
	logProbDE(i)=probprec;
end

[UniD,GammaD,PostGamD]=probordf(UniD,logProbDE, GammaD);
[MargGam]=marginale(GammaD, PostGamD, p);

disp(' ')
disp('--- End.') 
